﻿using System;
using IronPython.Hosting;
using Microsoft.Scripting;
using Microsoft.Scripting.Hosting;
using Microsoft.Scripting.Runtime;


namespace Hardcodet.Scripting
{
  class Class1
  {
    public static void TestScript()
    {
      string imports = "";
      imports += "import clr" + Environment.NewLine;
      imports += "clr.AddReference('System.Xml')" + Environment.NewLine;
      imports += "from System.Xml import *" + Environment.NewLine;

      string source ="";
      source += "value = value + 1";
      Console.Out.WriteLine(source);

      //init runtime / engine
      var runtime = Python.CreateRuntime();
      var engine = runtime.GetEngine("py");

      //a dictionary that receives the import simbols
      var importSymbols = new SymbolDictionary();

      //execute imports on our engine
      ScriptSource importSource = engine.CreateScriptSourceFromString(imports,
          SourceCodeKind.Statements);
      CompiledCode importCompiled = importSource.Compile();
      //create a scop to populate the symbols with the imported types
      ScriptScope importScope = engine.CreateScope(importSymbols);
      importCompiled.Execute(importScope);


      //compile worker script
      ScriptSource scriptSource = engine.CreateScriptSourceFromString(source,
        SourceCodeKind.Statements);
      CompiledCode workerScript = scriptSource.Compile();

    
      //create a new scope with the symbols of the import scope
      SymbolCache cache = new SymbolCache(importScope);
      ScriptScope run1 = engine.CreateScope(cache);
      //start with a value of 10
      run1.SetVariable("value", 10);
      workerScript.Execute(run1);

      //create a second scope with the symbols of the import scope
      cache = new SymbolCache(importScope);
      ScriptScope run2 = engine.CreateScope(cache);
      //start with a value of 20
      run2.SetVariable("value", 20);
      workerScript.Execute(run2);

      //both scopes actually share the same variable
      Console.Out.WriteLine(run1.GetVariable<int>("value"));
      Console.Out.WriteLine(run2.GetVariable<int>("value"));


      DateTime dt = DateTime.Now;
      //now loop 4000 times - create new scopes, but keep symbols!
      for (int i = 0; i < 10000; i++)
      {
        var scope = engine.CreateScope(new SymbolCache(importScope));
        scope.SetVariable("value", i);
        workerScript.Execute(scope);
      }

      double timespan = DateTime.Now.Subtract(dt).TotalMilliseconds;
      Console.Out.WriteLine("\n Total execution time: {0} ms.", timespan);
    }

    private static object syncRoot = new object();
  }
}
